# -*- coding: utf-8 -*-
import logging
import traceback
from os import getenv

from dotenv import load_dotenv
from kami_logging import benchmark_with, logging_with
from pydantic import validator

from kami_messenger.messenger import Messenger, RecipientFormatError
from kami_messenger.validator import DataValidator, PhoneFormatError

whatsapp_messenger_logger = logging.getLogger('Whatsapp Messenger')


class Whatsapp(Messenger):
    def _validate_messages_recipients(message):
        for recipient in message.recipients:
            try:
                data = DataValidator(recipient)
                data.isPhone()
            except PhoneFormatError:
                e = RecipientFormatError(
                    recipient,
                    f'Recipient {recipient} should be an valid phone number',
                )
                whatsapp_messenger_logger.error(f'{e.message} - {e.args}')
                raise
            except Exception as e:
                whatsapp_messenger_logger.error(traceback.format_exc())
            finally:
                return message

    @validator('messages', pre=True, each_item=True)
    @classmethod
    def recipientsValid(cls, message):
        cls._validate_message_recipients(message)

    @logging_with(whatsapp_messenger_logger)
    @benchmark_with(whatsapp_messenger_logger)
    def connect(self):
        try:
            engine = None
            # Implementar a conexão com o serviço do whatsapp e atualizar a variavel engine com o objeto responsavel por enviar mensagens
        except Exception as e:
            whatsapp_messenger_logger.error(traceback.format_exc())
            raise
        else:
            self.engine = engine
            whatsapp_messenger_logger.info(f'Success Connected')

    @logging_with(whatsapp_messenger_logger)
    @benchmark_with(whatsapp_messenger_logger)
    def _sendMessage(self, message):
        try:
            self.connect()
            # Implementar o envio de mensagens pelo botconversa usando o atributo 'engine'
            # Exemplo: self.engine.send_message()
            ...
        except Exception as e:
            whatsapp_messenger_logger.error(traceback.format_exc())
            raise
        finally:
            whatsapp_messenger_logger.info(
                f'Message Sucessufully Sent To {message.recipients}'
            )



# -*- coding: utf-8 -*-
from typing import Optional

from pydantic import BaseModel, root_validator, validator

from kami_messenger.validator import (
    DataValidator,
    EmailFormatError,
    PhoneFormatError,
)


class ContactMissingError(Exception):
    """Custom error that is raised when a contact doesn't have any of the contact methods."""

    def __init__(self, name: str, message: str) -> None:
        self.name = name
        self.message = message
        super().__init__(message)


class Contact(BaseModel):
    name: str
    email: Optional[str]
    phone: Optional[str]
    id_botconversa: Optional[str]

    @root_validator(pre=True)
    @classmethod
    def checkAtLeastOneContact(cls, values):
        """Make sure there is at least one of the contact fields options are defined."""

        if (
            'email' not in values
            and 'phone' not in values
            and 'id_botconversa' not in values
        ):
            raise ContactMissingError(
                name=values['name'],
                message='Contacts must have at least one of these contact methods [email, phone, id_botconversa]',
            )
        return values

    @validator('phone')
    @classmethod
    def phoneValid(cls, value):
        data = DataValidator(value)
        if not data.isPhone():
            raise PhoneFormatError(
                value=value, message="It's Not a Valid Phone Number"
            )
        return value

    @validator('email')
    @classmethod
    def emailValid(cls, value):
        data = DataValidator(value)
        if not data.isEmail():
            raise EmailFormatError(
                value=value, message="It's Not a Valid Email Address."
            )
        return value



# -*- coding: utf-8 -*-
import json
from os import getenv

from dotenv import load_dotenv
from pytest import mark

from kami_messenger.email_messenger import EmailMessenger

load_dotenv()


@mark.skip(reason='Needs to create fixtures to test without real values')
class TestEmailMessenger:
    data = f"""{{
      "name":"Gmail",
      "messages":[{{
          "sender":"dev@kamico.com.br",
          "recipients":["maicon@kamico.com.br"],
          "subject":"Teste",
          "body":"<p>Teste de mensagem</p>",
          "type":"text"
        }}],
      "credentials":{{
          "login":"{getenv("EMAIL_USER")}",
          "password":"{getenv("EMAIL_PASSWORD")}"
      }},
      "engine":""
    }}"""

    def test_when_email_get_valid_json_data_then_returns_new_email_messenger(
        self,
    ):
        json_data = json.loads(self.data)
        new_email_messenger = EmailMessenger(**json_data)
        assert json_data == new_email_messenger.dict()

    def test_when_email_sucess_connect_should_returns_200(self):
        json_data = json.loads(self.data)
        new_email_messenger = EmailMessenger(**json_data)
        status = new_email_messenger.connect()

        assert status == 200

    def test_when_connect_email_should_update_engine(self):
        json_data = json.loads(self.data)
        new_email_messenger = EmailMessenger(**json_data)
        new_email_messenger.connect()

        assert new_email_messenger.engine != None

    def test_when_send_message_by_email_should_return_sent_messages_quantity(
        self,
    ):
        json_data = json.loads(self.data)
        new_email_messenger = EmailMessenger(**json_data)
        messages_to_send = len(new_email_messenger.messages)
        sent_messages = new_email_messenger.sendMessage()

        assert messages_to_send == sent_messages



# -*- coding: utf-8 -*-
from email.message import EmailMessage
import logging
import mimetypes
import smtplib
import traceback

from kami_logging import benchmark_with, logging_with
from pydantic import validator

from kami_messenger.messenger import Messenger, RecipientFormatError
from kami_messenger.validator import DataValidator, EmailFormatError

email_messenger_logger = logging.getLogger('Email Messenger')


class EmailMessenger(Messenger):
    def _validate_message_recipients(message):
        for recipient in message.recipients:
            try:
                data = DataValidator(recipient)
                data.isEmail()
            except EmailFormatError:
                raise RecipientFormatError(
                    recipient,
                    f'Recipient {recipient} should be an valid e-mail address',
                )
            finally:
                return message

    @validator('messages', pre=True, each_item=True)
    @classmethod
    def recipientsValid(cls, message):
        cls._validate_message_recipients(message)

    @logging_with(email_messenger_logger)
    @benchmark_with(email_messenger_logger)
    def connect(self) -> int:
        try:
            engine = smtplib.SMTP('smtp.gmail.com: 587')
            engine.starttls()
            engine.login(
                self.credentials['login'], self.credentials['password']
            )
        except Exception as e:
            email_messenger_logger.error(traceback.format_exc())
            raise
        else:
            self.engine = engine
            email_messenger_logger.info(f'Success Connected')
            return 200

    @logging_with(email_messenger_logger)
    @benchmark_with(email_messenger_logger)
    def _sendMessage(self, message, attachments=[]) -> int:
        try:
            self.connect()

            email_message = EmailMessage()
            email_message['Subject'] = message.subject
            email_message['From'] = message.sender
            email_message['To'] = ', '.join(message.recipients)
            email_message.set_content(message.body, subtype='html')

            for attachment in attachments:
                with open(attachment, 'rb') as file:
                    file_data = file.read()
                    file_type, _ = mimetypes.guess_type(attachment)
                    if file_type is None:
                        file_type = 'application/octet-stream'
                    main_type, sub_type = file_type.split('/', 1)
                    email_message.add_attachment(
                        file_data,
                        maintype=main_type,
                        subtype=sub_type,
                        filename=attachment,
                    )

            self.engine.send_message(email_message)
        except Exception as e:
            raise e
        finally:
            email_messenger_logger.info(
                f'Message Successfully Sent To {", ".join(message.recipients)}'
            )
            return 200



# -*- coding: utf-8 -*-
from os.path import isfile, split
from typing import List

from jinja2 import Environment, FileSystemLoader
from jinja2 import Template as Jtemplate
from pydantic import BaseModel

from kami_messenger.contact import Contact
from kami_messenger.messenger import Message, Messenger


class KamiMessenger(BaseModel):
    messengers: List[Messenger]
    messages: List[Message]
    contacts: List[Contact]
    status: str = None

    def _setMessageTemplateFromFile(self, template):
        template_folder, template_file = (
            template_folder,
            template_file,
        ) = split(template)
        loader = FileSystemLoader(template_folder)
        enviroment = Environment(loader=loader)
        message_template = enviroment.get_template(template_file)
        self.message.body = message_template.render(self.contact)

    def _setMessageTemplateFromStr(self, template):
        message_template = Jtemplate(template)
        self.message.body = message_template.render(self.contact)

    def _getMessengerByName(self, messenger_name):
        return [
            messenger
            for messenger in self.messengers
            if messenger.name == messenger_name
        ][0]

    def setMessageTemplate(self, template):
        if isfile(template):
            self._setMessageTemplateFromFile(template)
        elif isinstance(template, str):
            self._setMessageTemplateFromFile(template)

    def setNewMessenger(self, messenger):
        if isinstance(messenger, Messenger):
            self.messengers.append(messenger)

    def sendMessageBy(self, messenger_name=None, messages=None):
        selected_messenger = self.messengers[0]
        selected_messages = self.messages

        if messenger_name:
            selected_messenger = self._getMessengerByName(messenger_name)

        if messages:
            selected_messages = messages

        selected_messenger.sendMessage(selected_messages)



from .messenger import (
    Message,
    MessageNotSendError,
    Messenger,
    RecipientFormatError,
)

__all__ = [
    'RecipientFormatError',
    'Message',
    'Messenger',
    'MessageNotSendError',
]



# -*- coding: utf-8 -*-
import json

from pytest import mark

from ..messenger import Message, Messenger


class TestMessage:
    data = """{
        "sender":"dev@kamico.com.br",
        "recipients":["maicon@kamico.com.br"],
        "subject":"Teste",
        "body":"<p>Teste de mensagem</p>",
        "type":"text"
    }"""

    def test_when_message_get_valid_json_data_then_returns_new_message(self):
        json_data = json.loads(self.data)
        new_message = Message(**json_data)
        assert json_data == new_message.dict()

    @mark.skip(reason='Defined but not implemented')
    def test_when_message_get_invalid_json_data_then_raise_exception(self):
        pass



# -*- coding: utf-8 -*-
import json

from dotenv import load_dotenv
from pytest import mark

from kami_messenger.botconversa import Botconversa

load_dotenv()


@mark.skip(reason='Defined but not implemented')
class TestBotconversa:
    data = f"""{{
      "name":"Botconversa",
      "messages":[{{
          "sender":"+5511916654692",
          "recipients":["+5521983144824"],
          "subject":"Teste",
          "body":"<p>Teste de mensagem</p>",
          "type":"text"
          }}],          
      "credentials":{{"api-key": "b6ba8d5c-19a1-4c38-8f5b-3966f11f2bbe"}},
      "engine":null
    }}"""

    def test_when_email_get_valid_json_data_then_returns_new_botconversa_messenger(
        self,
    ):
        json_data = json.loads(self.data)
        new_botconversa_messenger = Botconversa(**json_data)
        assert json_data == new_botconversa_messenger.dict()

    def test_when_send_message_by_botconversa_should_return_sent_messages_quantity(
        self,
    ):
        json_data = json.loads(self.data)
        new_botconversa_messenger = Botconversa(**json_data)
        messages_to_send = len(new_botconversa_messenger.messages)
        sent_messages = new_botconversa_messenger.sendMessage()

        assert messages_to_send == sent_messages



# -*- coding: utf-8 -*-

"""
The Messenger Module represents a generic messaging service, used to abstract the characteristics common to all messaging services used in the project.

Classes:
   Message: str Name that identifies the messaging service
   Messenger: List[Message] List of messages to send
   RecipientFormatError: Custom error that is raised when a recipients doesn't have the rigth format.
"""
from abc import ABC, abstractmethod
from typing import Any, Dict, List, Optional

from pydantic import BaseModel, validator


class RecipientFormatError(Exception):
    """
    Custom error that is raised when a recipients doesn't have the rigth format.

    Attributes:
        value: Exception value error
        message: Error message
    """

    def __init__(self, value: str, message: str) -> None:
        self.value = value
        self.message = message
        super().__init__(message)


class MessageNotSendError(Exception):
    """Custom error that is raised when a message is not sended to a recipient."""

    def __init__(self, value: str, message: str) -> None:
        self.value = value
        self.message = message
        super().__init__(message)


class Message(BaseModel):
    """
    Represents a generic message used in all messengers.

    Attributes:
        sender: Contact of the sender of the message
        recipients: List of contacts who will receive the message
        subject: Message subject is optional
        body: message body, content that will be sent to recipients
    """

    sender: str
    recipients: List[str]
    subject: Optional[str]
    body: str
    type: str = 'text'


class Messenger(ABC, BaseModel):
    """
    Represents a generic messaging service, used to abstract the characteristics common to all messaging services used in the project.

    Attributes:
        name: str Name that identifies the messaging service
        messages: List[Message] List of messages to send
        credentials: Dict Dictionary with the information needed to connect the service
        engine: Any Object that contains methods for sending messages
    """

    name: str
    messages: List[Message] = []
    credentials: Dict
    engine: Any = None

    @abstractmethod
    def _validate_message_recipients(message: Message) -> Message:
        """
        Checks whether the recipients of a given message are valid using validator module and pydantic validation especfications. The type of recipient changes according to the used messenger

        Args:
            message: Individual message to check recipients

        Returns:
            The given message if all recipients is valid

        Raises:
            RecipientFormatError: If at least one of the recipients is not valid.
        """
        ...

    @classmethod
    @abstractmethod
    def recipientsValid(cls, message: Message):
        """
        Calls _validate_message_recipients to check whether the recipients of a all messages in the messenger are valids. Uses pydantic validator decorator to iterate over all messages

        Args:
            message: Message to check recipients

        Raises:
            RecipientFormatError: If at least one of the recipients is not valid.
        """
        ...

    @abstractmethod
    def connect(self) -> int:
        """
        Connect with the messaging service using self.credentials for the necessary information to connect (eg Login and Password or API-Token) and updates the 'engine' attribute with the service created for sending messages if any success.

        Returns:
            An integer that represents the status code obeying the http service status code standard (eg: 200 to success and 404 to not found error).

        Raises:
            Exception: If there is any problem with the service trying to connect
        """
        ...

    @abstractmethod
    def _sendMessage(
        self, message: Message, attachments: List[str] = None
    ) -> int:
        """
        Helper method that implements sending a single message to its recipients

        Args:
            message: Individual message to sent to all recipients

        Returns:
            An integer that represents the status code obeying the http service status code standard (eg: 200 to success and 404 to not found error).

        Raises:
            Exception: If there is any problem with the service trying to send message.
        """
        ...

    def sendMessage(self, messages: List[Message] = None, attachments: List[str] = None) -> int:
        """
        Calls _sendMessage to send all messages from a list to their respective recipients, if it does not receive a list as a parameter it sends all messages belonging to the messages attribute

        Args:
            messages: A list of Individual messages to send, it's optional

        Returns:
            An integer that's represents the amount of sent messages

        Raises:
            Exception: If there is any problem with the service trying to send message.
        """

        sent_messages = 0
        selected_messages = self.messages
        if messages:
            selected_messages = messages

        for message in selected_messages:
            try:
                sent_messages += 1 if self._sendMessage(message, attachments) == 200 else 0
            except Exception as e:
                raise e

        return sent_messages



# -*- coding: utf-8 -*-
from html.parser import HTMLParser


class MyHTMLParser(HTMLParser):
    def __init__(self):
        super().__init__()
        self.start_tags = list()
        self.end_tags = list()
        self.attributes = list()

    def is_text_html(self):
        return len(self.start_tags) == len(self.end_tags)

    def handle_starttag(self, tag, attrs):
        self.start_tags.append(tag)
        self.attributes.append(attrs)

    def handle_endtag(self, tag):
        self.end_tags.append(tag)

    def handle_data(self, data):
        print('Encountered some data  :', data)



from .validator import (
    DataValidator,
    EmailFormatError,
    HTMLFormatError,
    IdBotconversaMissingError,
    PhoneFormatError,
)

__all__ = [
    'DataValidator',
    'HTMLFormatError',
    'PhoneFormatError',
    'EmailFormatError',
    'IdBotconversaMissingError',
]



import re
from os import getenv
from urllib.parse import urljoin

import phonenumbers as phonevalidator
import requests
from dotenv import load_dotenv

from .custom_html_parser import MyHTMLParser

load_dotenv()


class PhoneFormatError(Exception):
    """Custom error that is raised when a phone number doesn't have the rigth format."""

    def __init__(self, value: str, message: str) -> None:
        self.value = value
        self.message = message
        super().__init__(message)


class EmailFormatError(Exception):
    """Custom error that is raised when a email address doesn't have the rigth format."""

    def __init__(self, value: str, message: str) -> None:
        self.value = value
        self.message = message
        super().__init__(message)


class HTMLFormatError(Exception):
    """Custom error that is raised when a html document doesn't have the rigth format."""

    def __init__(self, value: str, message: str) -> None:
        self.value = value
        self.message = message
        super().__init__(message)


class IdBotconversaMissingError(Exception):
    """Custom error that is raised when a id to botconversa contact doesn't exist."""

    def __init__(self, value: str, message: str) -> None:
        self.value = value
        self.message = message
        super().__init__(message)


class DataValidator:
    def __init__(self, value: str):
        self.value = value

    def isPhone(self):
        phone = phonevalidator.parse(self.value)
        if not phonevalidator.is_valid_number(phone):
            raise PhoneFormatError(
                value=self.value, message="It's Not A Valid Phone Number."
            )

        return True

    def isEmail(self):
        regex = r'\b[A-Za-z0-9._%+-]+@[A-Za-z0-9.-]+\.[A-Z|a-z]{2,7}\b'
        if not (re.fullmatch(regex, self.value)):
            raise EmailFormatError(
                value=self.value, message="It's Not A Valid Email Address."
            )

        return True

    def isHtml(self):
        parser = MyHTMLParser()
        parser.feed(self.value)
        if not parser.is_text_html():
            raise HTMLFormatError(
                value=self.value, message="It's Not A Valid HTML document."
            )

        return True

    def _isIdBotconversa(self):
        api_url = f'https://backend.botconversa.com.br/api/v1/webhook/subscriber/get_by_phone/{self.value}/'

        headers = {
            'accept': 'application/json',
            'api-key': getenv('BOTCONVERSA_API_TOKEN'),
            'Content-Type': 'application/json',
        }
        response = requests.get(api_url, headers=headers)
        if response.status_code < 200 or response.status_code >= 400:
            raise IdBotconversaMissingError(
                value=self.value,
                message="It's Not A Valid IdBotconversa User.",
            )

        return True



# -*- coding: utf-8 -*-
import json
from os import getenv

from dotenv import load_dotenv
from pytest import mark

from kami_messenger.whatsapp import Whatsapp

load_dotenv()


@mark.skip(reason='Defined but not implemented')
class TestWhatsapp:
    data = f"""{{
      "name":"Whatsapp",
      "messages":[{{
          "sender":"",
          "recipients":["21983144824"],
          "subject":"Teste",
          "body":"<p>Teste de mensagem</p>"
        }}],
      "credentials":{{" implementar o dicionário das credenciais necessárias para acessar o whatsapp usando getenv() para proteção dos dados ": "valor"}},
      "engine":null
    }}"""

    def test_when_email_get_valid_json_data_then_returns_new_whatsapp_messenger(
        self,
    ):
        json_data = json.loads(self.data)
        new_whatsapp_messenger = Whatsapp(**json_data)
        assert json_data == new_whatsapp_messenger.dict()

    def test_when_whatsapp_sucess_connect_should_returns_200(self):
        json_data = json.loads(self.data)
        new_whatsapp_messenger = Whatsapp(**json_data)
        status = new_whatsapp_messenger.connect()

        assert status == 200

    def test_when_connect_whatsapp_should_update_engine(self):
        json_data = json.loads(self.data)
        new_whatsapp_messenger = Whatsapp(**json_data)
        new_whatsapp_messenger.connect()

        assert new_whatsapp_messenger.engine != None

    def test_when_send_message_by_whatsapp_should_return_sent_messages_quantity(
        self,
    ):
        json_data = json.loads(self.data)
        new_whatsapp_messenger = Whatsapp(**json_data)
        messages_to_send = len(new_whatsapp_messenger.messages)
        sent_messages = new_whatsapp_messenger.sendMessage()

        assert messages_to_send == sent_messages



from .contact import *
from .email_messenger import *
from .messenger import *
from .validator import *

__all__ = [
    email_messenger.__all__
    + messenger.__all__
    + contact.__all__
    + validator.__all__
]



# -*- coding: utf-8 -*-
import json
import logging
import traceback
from urllib.parse import urljoin

import requests
from dotenv import load_dotenv
from kami_logging import benchmark_with, logging_with
from pydantic import validator

from kami_messenger.messenger import (
    MessageNotSendError,
    Messenger,
    RecipientFormatError,
)
from kami_messenger.validator import DataValidator, IdBotconversaMissingError

botconversa_messenger_logger = logging.getLogger('Botconversa Messenger')
load_dotenv()


class Botconversa(Messenger):
    def _validate_message_recipients(self, message):
        for recipient in message.recipients:
            try:
                data = DataValidator(recipient)
                data._isIdBotconversa()
            except IdBotconversaMissingError:
                e = RecipientFormatError(
                    recipient,
                    f'Recipient {recipient} should be an valid botconversa contact',
                )
                botconversa_messenger_logger.error(f'{e.message} - {e.args}')
                raise MessageNotSendError
            except Exception as e:
                botconversa_messenger_logger.error(traceback.format_exc())
            finally:
                return message

    @validator('messages', pre=True, each_item=True)
    @classmethod
    def recipientsValid(cls, message):
        cls._validate_messages_recipients(message)

    @logging_with(botconversa_messenger_logger)
    @benchmark_with(botconversa_messenger_logger)
    def connect(self):
        try:
            engine = None
            # Implementar a conexão com o serviço do botconversa e atualizar a variavel engine com o objeto responsavel por enviar mensagens
        except Exception as e:
            botconversa_messenger_logger.error(traceback.format_exc())
            raise
        else:
            self.engine = engine
            botconversa_messenger_logger.info(f'Success Connected')

    @logging_with(botconversa_messenger_logger)
    @benchmark_with(botconversa_messenger_logger)
    def _getIdBotconversaByPhone(self, phone: str) -> str:
        url = f'https://backend.botconversa.com.br/api/v1/webhook/subscriber/get_by_phone/{phone}/'

        headers = {
            'accept': 'application/json',
            'api-key': self.credentials['api-key'],
            'Content-Type': 'application/json',
        }

        try:
            response = requests.get(url, headers=headers)
            if response.status_code < 200 or response.status_code > 299:
                raise MessageNotSendError(
                    value=response.status_code,
                    message=f'API Request returns {response.status_code}',
                )

            res_content = json.loads(response.text)
            return res_content['id']
        except Exception as e:
            botconversa_messenger_logger.error(traceback.format_exc())
            raise
        ...

    @logging_with(botconversa_messenger_logger)
    @benchmark_with(botconversa_messenger_logger)
    def _sendMessage(self, message):
        try:
            message_data = {'type': message.type, 'value': message.body}
            for recipient in message.recipients:
                id_botconversa = self._getIdBotconversaByPhone(recipient)
                url = f'https://backend.botconversa.com.br/api/v1/webhook/subscriber/{id_botconversa}/send_message/'

                headers = {
                    'accept': 'application/json',
                    'api-key': self.credentials['api-key'],
                    'Content-Type': 'application/json',
                }
                response = requests.post(
                    url, headers=headers, data=json.dumps(message_data)
                )
                if response.status_code < 200 or response.status_code > 299:
                    raise MessageNotSendError(
                        value=response.status_code,
                        message=f'API Request returns {response.status_code}',
                    )
                botconversa_messenger_logger.info(
                    f'Send message to: {recipient} | Status: {response.status_code}'
                )
                return 200
        except Exception as e:
            botconversa_messenger_logger.error(traceback.format_exc())
            raise
        finally:
            botconversa_messenger_logger.info(
                f'Message Sucessufully Sent To {message.recipients}'
            )



# -*- coding: utf-8 -*-
import json

from pytest import mark

from ..contact import Contact


class TestContact:
    json_data = """{
    "name":"Test1",
    "email":"test@test.com",
    "phone":"+5511999999999",
    "id_botconversa":"123574"
  }"""

    def test_when_contact_get_valid_json_data_then_returns_new_contact(self):
        contact_json_data = json.loads(self.json_data)
        new_contact = Contact(**contact_json_data)
        assert contact_json_data == new_contact.dict()

    @mark.skip(reason='Defined but not implemented')
    def test_when_contact_get_invalid_json_data_then_returns_none(self):
        ...
